==> inspect_ai-0.3.41.tar.gz: inspect_ai-0.3.41/examples/approval/README.md <==
# Approval Mode Demo

## Introduction

This is a demonstrate of Inspect's [Approval Mode](https://inspect.ai-safety-institute.org.uk/approval.html) whereby tool calls are approved by a combination of approvers and approval policies.

To run this demonstration, specify the approval configuration file and the `--trace` option to see model interactions printed to the terminal:

``` bash
cd examples/approval
inspect eval approval.py --approval approval.yaml --trace
```

You can also run the approval script directly and it will run the task in the appropriate approval and trace modes:

``` bash
python3 examples/approval/approval.py
```

## Task Setup

The task we'll run asks the model to demonstrate running some bash and python commands in a Docker container:

```python
@task
def approval_demo() -> Task:
    return Task(
        dataset=[
            Sample(
                input="Please use the bash tool to demonstrate the use of the bash ls command, then demonstrate the use of the bash rm command."
            ),
            Sample(
                input="Please use the python tool to the use of the Python print function, then demonstrate the math.factorial function, then demonstrate the use of the shutil.rmtree function."
            ),
        ],
        solver=[
            system_message(
                "You will ba asked to demonstrate various uses of the bash and python tools. Please make only one tool call at a time rather than attempting to demonstrate multiple uses in a single call."
            ),
            use_tools(bash(), python()),
            generate(),
        ],
        sandbox="docker",
    )
```

## Approval Policy

We'll evaluate this task using the approval policy defined in `approval.yaml`:

```bash
inspect eval approval.py --approval approval.yaml --trace
```

Here is the approval configuration:

```yaml
approvers:
  - name: bash_allowlist
    tools: "*bash*"
    allowed_commands: ["ls", "echo", "cat"]

  - name: python_allowlist
    tools: "*python*"
    allowed_functions: ["print"]
    allowed_modules: ["math"]

  - name: human
    tools: "*"
```

The list of approvers is applied in order and bound to the tools that match the globs in the `tools` configuration. Note that the `bash_allowlist` and `python_allowlist` approvers are custom approvers defined in the `approval.py` source file (they aren't included in Inspect). These approvers will make one of the following approval decisions for each tool call they are configured to handle:

1) Allow the tool call (based on the various configured options)
2) Disallow the tool call (because it is considered dangerous under all conditions)
3) Escalate the tool call to the human approver.

Note that the human approver is last and is bound to all tools, so escalations from the bash and python allowlist approvers will end up prompting the human approver.

## Custom Approvers

Inspect includes two built-an approvers: `human` for interactive approval at the terminal and `auto` for automatically approving or rejecting specific tools. The code above uses two custom approvers, which you can see the source code of in [approval.py](./approval.py). Here is the basic form of a custom approver:

```python
@approver
def bash_allowlist(
    allowed_commands: list[str],
    allow_sudo: bool = False,
    command_specific_rules: dict[str, list[str]] | None = None,
) -> Approver:
    """Create an approver that checks if a bash command is in an allowed list."""

    async def approve(
        message: str,
        call: ToolCall,
        view: ToolCallView,
        state: TaskState | None = None,
    ) -> Approval:

        # Make approval decision
        
        ...

    return approve
```


See the documentation on [Approval Mode](https://inspect.ai-safety-institute.org.uk/approval.html) for additional information on using approvals and defining custom approvers.

==> inspect_ai-0.3.58.tar.gz: inspect_ai-0.3.58/examples/approval/README.md <==
# Approval Mode Demo

## Introduction

This is a demonstrate of Inspect's [Approval Mode](https://inspect.ai-safety-institute.org.uk/approval.html) whereby tool calls are approved by a combination of approvers and approval policies.

To run this demonstration, specify the approval configuration file as follows:

``` bash
cd examples/approval
inspect eval approval.py --approval approval.yaml
```

## Task Setup

The task we'll run asks the model to demonstrate running some bash and python commands in a Docker container:

```python
@task
def approval_demo() -> Task:
    return Task(
        dataset=[
            Sample(
                input="Please use the bash tool to demonstrate the use of the bash ls command, then demonstrate the use of the bash rm command."
            ),
            Sample(
                input="Please use the python tool to the use of the Python print function, then demonstrate the math.factorial function, then demonstrate the use of the shutil.rmtree function."
            ),
        ],
        solver=[
            system_message(
                "You will ba asked to demonstrate various uses of the bash and python tools. Please make only one tool call at a time rather than attempting to demonstrate multiple uses in a single call."
            ),
            use_tools(bash(), python()),
            generate(),
        ],
        sandbox="docker",
    )
```

## Approval Policy

We'll evaluate this task using the approval policy defined in `approval.yaml`:

```bash
inspect eval approval.py --approval approval.yaml
```

Here is the approval configuration:

```yaml
approvers:
  - name: bash_allowlist
    tools: "*bash*"
    allowed_commands: ["ls", "echo", "cat"]

  - name: python_allowlist
    tools: "*python*"
    allowed_functions: ["print"]
    allowed_modules: ["math"]

  - name: human
    tools: "*"
```

The list of approvers is applied in order and bound to the tools that match the globs in the `tools` configuration. Note that the `bash_allowlist` and `python_allowlist` approvers are custom approvers defined in the `approval.py` source file (they aren't included in Inspect). These approvers will make one of the following approval decisions for each tool call they are configured to handle:

1) Allow the tool call (based on the various configured options)
2) Disallow the tool call (because it is considered dangerous under all conditions)
3) Escalate the tool call to the human approver.

Note that the human approver is last and is bound to all tools, so escalations from the bash and python allowlist approvers will end up prompting the human approver.

## Custom Approvers

Inspect includes two built-an approvers: `human` for interactive approval at the terminal and `auto` for automatically approving or rejecting specific tools. The code above uses two custom approvers, which you can see the source code of in [approval.py](./approval.py). Here is the basic form of a custom approver:

```python
@approver
def bash_allowlist(
    allowed_commands: list[str],
    allow_sudo: bool = False,
    command_specific_rules: dict[str, list[str]] | None = None,
) -> Approver:
    """Create an approver that checks if a bash command is in an allowed list."""

    async def approve(
        message: str,
        call: ToolCall,
        view: ToolCallView,
        state: TaskState | None = None,
    ) -> Approval:

        # Make approval decision
        
        ...

    return approve
```


See the documentation on [Approval Mode](https://inspect.ai-safety-institute.org.uk/approval.html) for additional information on using approvals and defining custom approvers.
